@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2017 by Hans-Peter Deifel                               *
+ *   Copyright (C) 2010-2018 by Hans-Peter Deifel                          *
  *   hpd@hpdeifel.de                                                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -59,6 +59,7 @@
 #include "regengine.h"
 #include "search.h"
 #include "cache.h"
+#include "intervals.h"
 
 using namespace std;
 
@@ -79,10 +80,12 @@ enum {
 	WARN_EMPTY_OPTION,
 	UNAC_OPTION,
 	CACHE_OPTION,
+	PAGE_RANGE_OPTION,
 };
 
 struct option long_options[] =
 {
+	// name, has_arg, *flag, val
 	{"ignore-case", 0, 0, 'i'},
 	{"perl-regexp", 0, 0, 'P'},
 	{"page-number", 0, 0, 'n'},
@@ -111,22 +114,27 @@ struct option long_options[] =
 	{"after-context", 1, 0, 'A'},
 	{"before-context", 1, 0, 'B'},
 	{"context", 1, 0, 'C'},
+	{"page-range", 1, 0, PAGE_RANGE_OPTION},
+	{"regexp", 1, 0, 'e'},
+	{"file", 1, 0, 'f'},
+	{"files-with-matches", 0, 0, 'l'},
+	{"files-without-match", 0, 0, 'L'},
 	{0, 0, 0, 0}
 };
 
 #ifdef HAVE_UNAC
 /* convenience layer over libunac. */
-static char *simple_unac(const Options &opts, char *string)
+string simple_unac(const Options &opts, const string str)
 {
 	if (!opts.use_unac)
-		return string;
+		return str;
 
 	char *res = NULL;
 	size_t reslen = 0;
 
-	if (unac_string("UTF-8", string, strlen(string), &res, &reslen)) {
+	if (unac_string("UTF-8", str.c_str(), str.size(), &res, &reslen)) {
 		perror("pdfgrep: Failed to remove accents: ");
-		return strdup(string);
+		return str;
 	}
 
 	return res;
@@ -230,23 +238,25 @@ static void print_help(char *self)
 	     << "Search for PATTERN in each FILE." << endl
 	     << "PATTERN is, by default, an extended regular expression." << endl
 	     << endl
-	     << "Options:" << endl
-	     << " -i, --ignore-case\t\tIgnore case distinctions" << endl
-	     << " -P, --pcre\t\t\tUse Perl compatible regular expressions (PCRE)" << endl
-	     << " -H, --with-filename\t\tPrint the file name for each match" << endl
-	     << " -h, --no-filename\t\tSuppress the prefixing of file name on output" << endl
-	     << " -n, --page-number\t\tPrint page number with output lines" << endl
-	     << " -c, --count\t\t\tPrint only a count of matches per file" << endl
-	     << "     --color WHEN\t\tUse colors for highlighting;" << endl
-	     << "\t\t\t\tWHEN can be `always', `never' or `auto'" << endl
-	     << " -p, --page-count\t\tPrint only a count of matches per page" << endl
-	     << " -m, --max-count NUM\t\tStop reading after NUM matching lines (per file)" << endl
-	     << " -q, --quiet\t\t\tSuppress normal output" << endl
-	     << " -r, --recursive\t\tSearch directories recursively" << endl
-	     << " -R, --dereference-recursive\tLikewise, but follow all symlinks" << endl
-	     << "     --cache\t\tUse cache for faster operation" << endl
-	     << "     --help\t\t\tPrint this help" << endl
-	     << " -V, --version\t\t\tShow version information" << endl;
+	     << "Commonly used options:" << endl
+	     << " -i, --ignore-case              Ignore case distinctions" << endl
+	     << " -P, --pcre                     Use Perl compatible regular expressions (PCRE)" << endl
+	     << " -H, --with-filename            Print the file name for each match" << endl
+	     << " -h, --no-filename              Suppress the prefixing of file name on output" << endl
+	     << " -n, --page-number              Print page number with output lines" << endl
+	     << " -c, --count                    Print only a count of matches per file" << endl
+	     << "     --color WHEN               Use colors for highlighting;" << endl
+	     << "                                WHEN can be `always', `never' or `auto'" << endl
+	     << " -p, --page-count               Print only a count of matches per page" << endl
+	     << " -m, --max-count NUM            Stop reading after NUM matching lines (per file)" << endl
+	     << " -q, --quiet                    Suppress normal output" << endl
+	     << " -r, --recursive                Search directories recursively" << endl
+	     << " -R, --dereference-recursive    Likewise, but follow all symlinks" << endl
+	     << "     --cache                    Use cache for faster operation" << endl
+	     << "     --help                     Print this help" << endl
+	     << " -V, --version                  Show version information" << endl << endl
+	     << "The above list is only a selection of commonly used options. Please refer" << endl
+	     << "to the man page for a complete list." << endl;
 }
 
 static void print_version()
@@ -254,7 +264,7 @@ static void print_version()
 	cout << "This is " << PACKAGE << " version " << VERSION << "." << endl << endl;
 	cout << "Using poppler version " << poppler::version_string().c_str() << endl;
 #ifdef HAVE_UNAC
-	cout << "Using unac version " << unac_version() << endl;
+	cout << "Using libunac version " << unac_version() << endl;
 #endif
 #ifdef HAVE_LIBPCRE
 	cout << "Using libpcre version " << pcre_version() << endl;
@@ -410,6 +420,28 @@ static bool parse_int(const char *str, int *i)
 	return true;
 }
 
+bool read_pattern_file(string filename, vector<string> &patterns)
+{
+	ifstream file(filename);
+
+	if (!file.is_open()) {
+		err() << filename << ": " << strerror(errno) << endl;
+		return false;
+	}
+
+	string line;
+	while (getline(file, line)) {
+		patterns.push_back(line);
+	}
+
+	if (file.bad()) {
+		err() << filename << ": " << strerror(errno) << endl;
+		return false;
+	}
+
+	return true;
+}
+
 #if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 29
 static void handle_poppler_errors(const string &msg, void *_opts)
 {
@@ -447,8 +479,12 @@ int main(int argc, char** argv)
 		COLOR_NEVER
 	} use_colors = COLOR_AUTO;
 
+	// patterns specified with --regex or --file
+	vector<string> patterns;
+	bool patterns_specified = false;
+
 	while (1) {
-		int c = getopt_long(argc, argv, "icA:B:C:nrRhHVPpqm:FoZ",
+		int c = getopt_long(argc, argv, "icA:B:C:nrRhHVPpqm:FoZe:f:lL",
 				long_options, NULL);
 
 		if (c == -1)
@@ -603,6 +639,29 @@ int main(int argc, char** argv)
 				options.outconf.context_mode = true;
 				break;
 
+			case PAGE_RANGE_OPTION:
+				options.page_range = IntervalContainer::fromString(optarg);
+				break;
+
+			case 'e':
+				patterns_specified = true;
+				patterns.push_back(string(optarg));
+				break;
+
+			case 'f':
+				patterns_specified = true;
+				if (!read_pattern_file(string(optarg), patterns))
+					exit(EXIT_ERROR);
+				break;
+
+			case 'l':
+				options.only_filenames = OnlyFilenames::WITH_MATCHES;
+				break;
+
+			case 'L':
+				options.only_filenames = OnlyFilenames::WITHOUT_MATCH;
+				break;
+
 			/* In these two cases, getopt already prints an
 			 * error message
 			 */
@@ -615,30 +674,49 @@ int main(int argc, char** argv)
 		}
 	}
 
-	if (argc == optind || (argc - optind < 2 && options.recursive == Recursion::NONE)) {
+	int remaining_args = argc - optind;
+	int required_args = 0;
+	if (!patterns_specified) required_args++;
+	if (options.recursive == Recursion::NONE) required_args++;
+
+	if (remaining_args < required_args) {
 		print_usage(argv[0]);
 		exit(EXIT_ERROR);
 	}
 
-	char *pattern = argv[optind++];
-#ifdef HAVE_UNAC
-	pattern = simple_unac(options, pattern);
-#endif
-
 	unique_ptr<Regengine> re;
 	if (re_engine == (RE_FIXED | RE_PCRE)) {
 		err() << "--pcre and --fixed cannot be used together" << endl;
 		exit(EXIT_ERROR);
 	}
+
+	auto make_regengine = [&](const string pattern) -> unique_ptr<Regengine> {
+#ifdef HAVE_UNAC
+		const string new_pattern = simple_unac(options, pattern);
+#else
+		const string new_pattern = move(pattern);
+#endif
 #ifdef HAVE_LIBPCRE
-	if (re_engine == RE_PCRE) {
-		re.reset(new PCRERegex(pattern, options.ignore_case));
-	} else
+		if (re_engine == RE_PCRE) {
+			return unique_ptr<PCRERegex>(new PCRERegex(new_pattern, options.ignore_case));
+		} else
 #endif // HAVE_LIBPCRE
-	if (re_engine == RE_FIXED) {
-		re.reset(new FixedString(pattern, options.ignore_case));
+		if (re_engine == RE_FIXED) {
+			return unique_ptr<FixedString>(new FixedString(new_pattern, options.ignore_case));
+		} else {
+			return unique_ptr<PosixRegex>(new PosixRegex(new_pattern, options.ignore_case));
+		}
+
+	};
+
+	if (patterns.empty()) {
+		re = make_regengine(argv[optind++]);
 	} else {
-		re.reset(new PosixRegex(pattern, options.ignore_case));
+		auto patt_list = std::unique_ptr<PatternList>(new PatternList());
+		for (auto p : patterns) {
+			patt_list->add_pattern(make_regengine(p));
+		}
+		re = move(patt_list);
 	}
 
 #if POPPLER_VERSION_MAJOR > 0 || POPPLER_VERSION_MINOR >= 29
@@ -671,6 +749,9 @@ int main(int argc, char** argv)
 		options.outconf.context_mode = false;
 	}
 
+	// TODO Warn about --files-{with-matches,without-match} and other output
+	// options
+
 	if (excludes_empty(options.includes))
 		exclude_add(options.includes, "*.pdf");
 
