@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2017 by Hans-Peter Deifel                               *
+ *   Copyright (C) 2015-2018 by Hans-Peter Deifel                          *
  *   hpd@hpdeifel.de                                                       *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -23,6 +23,7 @@
 
 #include "config.h"
 #include "exclude.h"
+#include "intervals.h"
 
 #include <vector>
 #include <string>
@@ -67,6 +68,11 @@ struct Outconf {
 	Colorconf colors;
 };
 
+enum class OnlyFilenames {
+	NOPE,
+	WITH_MATCHES,
+	WITHOUT_MATCH
+};
 
 struct Options {
 	bool ignore_case = false;
@@ -87,6 +93,17 @@ struct Options {
 	ExcludeList includes;
 	bool use_cache = false;
 	std::string cache_directory;
+	IntervalContainer page_range;
+	OnlyFilenames only_filenames = OnlyFilenames::NOPE;
 };
 
+#ifdef HAVE_UNAC
+/* convenience layer over libunac */
+std::string simple_unac(const Options &opts, const std::string str);
+#endif
+
 #endif /* PDFGREP_H */
+
+/* Local Variables: */
+/* mode: c++ */
+/* End: */
